from werewolf_players import No_player


class Role:
	def __init__(self, game, player=No_player()):
		self.game = game
		self.player = player
		self.player.setRole(self)
		self.add_yourself()
		self.is_Hunter = self.is_Tanner = self.is_Werewolf = self.is_Minion = False

	def add_yourself(self):
		self.game.role[type(self)] = self

	async def send_role_list(self, cls):
		await self.player.send_info(f"{cls}: {', '.join(str(p) for p in self.game.role[cls])}")

	@staticmethod
	def match(message):
		for role_class in Role.__subclasses__():
			if message.casefold() == role_class.name():
				return role_class
		raise ValueError(f"Invalid role: {message}")

	@classmethod
	def name(cls):
		return cls.__name__.casefold()

	def __str__(self):
		return self.name()


class Doppelganger(Role):
	pass


class Werewolf(Role):
	def __init__(self, game, player=No_player()):
		super().__init__(game, player)
		self.is_Werewolf = True

	def add_yourself(self):
		self.game.role[Werewolf].append(self)

	async def phase(self):
		if len(self.game.role[Werewolf]) >= 2:
			await self.send_role_list(Werewolf)
		else:
			await self.player.send_info("You are the only werewolf")
			self.choice = await self.player.get_choice("Which card in the middle do you want to look at?", ["left", "middle", "right"])  # 0, 1, 2

			await self.player.send_info(f"A card in the middle is: {self.game.middle_card[self.choice].name()}")


class Minion(Role):
	def __init__(self, game, player=No_player()):
		super().__init__(game, player)
		self.is_Minion = True

	async def send_info(self):
		if len(self.game.werewolf_list) == 0:
			await self.player.send_info("There were no werewolves, so you need to kill a villager!")
		else:
			await self.send_role_list(Werewolf)


class Mason(Role):
	def add_yourself(self):
		self.game.role[Mason].append(self)

	async def send_info(self):
		await self.send_role_list(Mason)


class Seer(Role):
	async def query(self):
		self.choice = await self.player.get_choice("Which 1 player card or 2 middle cards do you want to look at?", self.player.other() + ["left & middle", "middle & right", "left & right"])

	async def send_info(self):
		if self.choice < len(self.player.other()):
			await self.player.send_info(self.player.other()[self.choice].night_role)
		else:
			self.choice -= len(self.player.other())
			if self.choice == 0:
				a, b = 0, 1
			elif self.choice == 1:
				a, b = 1, 2
			else:
				a, b = 0, 2

			await self.player.send_info(f"{self.game.middle_card[a]} {self.game.middle_card[b]}")


class Robber(Role):
	async def query(self):
		self.choice = await self.player.get_choice("Which player do you want to rob?", self.player.other())

	async def simulate(self):
		self.player.swap(self.player.other()[self.choice])

	async def send_info(self):
		await self.player.send_info(f"You robbed: {self.player.day_role}")


class Troublemaker(Role):
	async def query(self):
		self.A, self.B = await self.player.get_double_choice("Who do you want to exchange? (send two numbers)", self.player.other())

	async def simulate(self):
		self.player.other()[self.A].swap(self.player.other()[self.B])


class Drunk(Role):
	async def query(self):
		self.choice = await self.player.get_choice("Which card from the middle do you want to take?", ["left", "middle", "right"])

	async def simulate(self):
		self.player.day_role, self.game.middle_card[self.choice] = self.game.middle_card[self.choice], self.player.day_role  # swap


class Insomniac(Role):
	async def send_info(self):
		await self.player.send_info(f"You are now: {self.player.day_role}")


class Villager(Role):
	pass


class Tanner(Role):
	def __init__(self, game, player=No_player()):
		super().__init__(game, player)
		self.is_Tanner = True


class Hunter(Role):
	def __init__(self, game, player=No_player()):
		super().__init__(game, player)
		self.is_Hunter = True


class No_role(Role):
	pass
